package me.unurled.sacredrealms.sr.components.player;

import static me.unurled.sacredrealms.sr.utils.Items.lore;
import static me.unurled.sacredrealms.sr.utils.Logger.warn;

import com.google.gson.annotations.Expose;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import me.unurled.sacredrealms.sr.components.attributes.Attribute;
import me.unurled.sacredrealms.sr.components.item.Item;
import me.unurled.sacredrealms.sr.events.player.PlayerKillEvent;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.entity.EntityDamageEvent.DamageCause;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataType;
import org.bukkit.potion.PotionEffect;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class SRPlayer {

  private static final List<Long> levelRequirements =
      List.of(100L, 200L, 400L, 800L, 1600L, 3200L, 6400L, 12800L, 25600L, 51200L);

  @Expose private UUID uuid = null;
  @Expose private int level = 1;
  @Expose private Long experience = 0L;
  private double health = 100;
  @Expose private HashMap<Attribute, Double> attributes = new HashMap<>();
  private HashMap<Attribute, HashMap<ItemStack, Double>> itemAttributes = new HashMap<>();
  @Expose private List<PotionEffect> potionEffects = new ArrayList<>();
  @Expose private Inventory inventory = null;

  public SRPlayer(@NotNull UUID uuid) {
    this.uuid = uuid;
  }

  @NotNull
  public UUID getUuid() {
    return uuid;
  }

  @NotNull
  public Inventory getInventory() {
    return inventory;
  }

  public void setInventory(Inventory inventory) {
    this.inventory = inventory;
  }

  @NotNull
  public Map<Attribute, Double> getAttributes() {
    return attributes;
  }

  public double getAttribute(@NotNull Attribute attribute) {
    return getAttribute(attribute, attribute.getDefaultValue());
  }

  @NotNull
  public Double getAttribute(@NotNull Attribute attribute, @NotNull Double defaultValue) {
    return attributes.getOrDefault(attribute, defaultValue);
  }

  public void setAttribute(@NotNull Attribute attribute, @NotNull Double value) {
    attributes.put(attribute, value);
  }

  public void removeAttribute(@NotNull Attribute attribute) {
    attributes.remove(attribute);
  }

  @Nullable
  public Object getHandItemAttribute(@NotNull Attribute attribute) {
    Player p = Bukkit.getPlayer(uuid);
    if (p == null) {
      return null;
    }
    Object o =
        p.getInventory()
            .getItemInMainHand()
            .getItemMeta()
            .getPersistentDataContainer()
            .get(attribute.getKey(), attribute.getType());
    if (o == null) {
      setHandItemAttribute(attribute, attribute.getDefaultValue());
      return attribute.getDefaultValue();
    }
    return o;
  }

  public void setHandItemAttribute(@NotNull Attribute attribute, @NotNull Double value) {
    Player p = Bukkit.getPlayer(uuid);
    if (p == null) {
      return;
    }
    if (checkIfValueIsOutOfBoundsForAttribute(attribute, value, p)) return;
    ItemStack i = p.getInventory().getItemInMainHand();
    ItemMeta im = i.getItemMeta();
    im.getPersistentDataContainer().set(attribute.getKey(), PersistentDataType.DOUBLE, value);
    i.setItemMeta(im);
    lore(i);
    p.getInventory().setItemInMainHand(i);
  }

  private boolean checkIfValueIsOutOfBoundsForAttribute(
      @NotNull Attribute attribute, @NotNull Double value, Player p) {
    if (!attribute.getType().getComplexType().isInstance(value)) {
      return true;
    }
    if (attribute.getType().equals(PersistentDataType.DOUBLE)) {
      double d = value;
      if (d < attribute.getMinValue() || d > attribute.getMaxValue()) {
        warn(
            "(Player "
                + p.getName()
                + ") Value "
                + d
                + " for attribute "
                + attribute.getID()
                + " is out of bounds. Min: "
                + attribute.getMinValue()
                + " Max: "
                + attribute.getMaxValue()
                + " Value: "
                + d);
        return true;
      }
    }
    return false;
  }

  public void removeHandItemAttribute(@NotNull Attribute attribute) {
    Player p = Bukkit.getPlayer(uuid);
    if (p == null) {
      return;
    }
    ItemStack i = p.getInventory().getItemInMainHand();
    ItemMeta im = i.getItemMeta();
    im.getPersistentDataContainer().remove(attribute.getKey());
    i.setItemMeta(im);
  }

  @SuppressWarnings("unused")
  public void addPotionEffect(@NotNull PotionEffect effect) {
    potionEffects.add(effect);
  }

  @SuppressWarnings("unused")
  public void removePotionEffect(@NotNull PotionEffect effect) {
    potionEffects.remove(effect);
  }

  @SuppressWarnings("unused")
  public void clearPotionEffects() {
    potionEffects.clear();
  }

  @NotNull
  public List<PotionEffect> getPotionEffects() {
    if (potionEffects == null) potionEffects = new ArrayList<>();
    return potionEffects;
  }

  public void setPotionEffects(@NotNull List<PotionEffect> effects) {
    potionEffects.clear();
    potionEffects.addAll(effects);
  }

  public void setItemAttributes(
      @NotNull Attribute attribute, @NotNull ItemStack item, @NotNull Double value) {
    if (itemAttributes == null) itemAttributes = new HashMap<>();
    if (itemAttributes.containsKey(attribute)) {
      itemAttributes.get(attribute).put(item, value);
    } else {
      HashMap<ItemStack, Double> map = new HashMap<>();
      map.put(item, value);
      itemAttributes.put(attribute, map);
    }
  }

  public void removeItemAttributes(@NotNull Attribute attribute, @NotNull ItemStack item) {
    if (itemAttributes == null) itemAttributes = new HashMap<>();
    if (itemAttributes.containsKey(attribute)) {
      itemAttributes.get(attribute).remove(item);
    }
  }

  @SuppressWarnings("unused")
  public @Nullable Double getItemAttributes(@NotNull Attribute attribute, @NotNull ItemStack item) {
    if (itemAttributes == null) itemAttributes = new HashMap<>();
    if (itemAttributes.containsKey(attribute)) {
      return itemAttributes.get(attribute).get(item);
    }
    return null;
  }

  @SuppressWarnings("unused")
  public @NotNull Map<ItemStack, Double> getItemAttributes(@NotNull Attribute attribute) {
    if (itemAttributes == null) itemAttributes = new HashMap<>();
    return itemAttributes.getOrDefault(attribute, new HashMap<>());
  }

  @SuppressWarnings("unused")
  public @NotNull Map<Attribute, Double> getItemAttributes(Item item) {
    if (itemAttributes == null) itemAttributes = new HashMap<>();
    HashMap<Attribute, Double> map = new HashMap<>();
    itemAttributes.forEach((k, v) -> map.put(k, v.getOrDefault(item.toItemStack(), 0d)));
    return map;
  }

  public @NotNull Double getTotalItemAttribute(@NotNull Attribute attribute) {
    if (itemAttributes == null) itemAttributes = new HashMap<>();
    return itemAttributes.getOrDefault(attribute, new HashMap<>()).values().stream()
        .reduce(0.0, Double::sum);
  }

  public int getLevel() {
    return level;
  }

  public void setLevel(int i) {
    level = i;
  }

  public double getHealth() {
    return health;
  }

  public void setHealth(double health, DamageCause cause) {
    if (health < 0) {
      health = 0;
      // kill player
      PlayerKillEvent event = new PlayerKillEvent(this, cause);
      Bukkit.getPluginManager().callEvent(event);
    }
    this.health = health;
  }

  public Long getExperience() {
    return experience;
  }

  public void setExperience(Long experience) {
    // checks if experience is above next level requirements
    if (experience >= levelRequirements.get(level - 1)) {
      PlayerManager pm = PlayerManager.getInstance(PlayerManager.class);
      setLevel(level + 1);
      pm.levelUp(this, level - 1);
    }
    this.experience = experience;
  }
}
